import logging
import os
from six import with_metaclass

from kalliope.core.Models.RecognitionOptions import RecognitionOptions
from .YAMLLoader import YAMLLoader
from kalliope.core.Models.Resources import Resources
from kalliope.core.Utils.Utils import Utils
from kalliope.core.Models import Singleton
from kalliope.core.Models.RestAPI import RestAPI
from kalliope.core.Models.Settings import Settings
from kalliope.core.Models.Stt import Stt
from kalliope.core.Models.Trigger import Trigger
from kalliope.core.Models.Player import Player
from kalliope.core.Models.Tts import Tts
from kalliope.core.Utils.FileManager import FileManager

FILE_NAME = "settings.yml"

logging.basicConfig()
logger = logging.getLogger("kalliope")


class SettingInvalidException(Exception):
    """
    Some data must match the expected value/type

    .. seealso:: Settings
    """
    pass


class NullSettingException(Exception):
    """
    Some Attributes can not be Null

    .. seealso:: Settings
    """
    pass


class SettingNotFound(Exception):
    """
    Some Attributes are missing

    .. seealso:: Settings
    """
    pass


class SettingLoader(with_metaclass(Singleton, object)):
    """
    This Class is used to get the Settings YAML and the Settings as an object
    """

    def __init__(self, file_path=None):
        self.file_path = file_path
        if self.file_path is None:
            self.file_path = Utils.get_real_file_path(FILE_NAME)
        else:
            self.file_path = Utils.get_real_file_path(file_path)
        # if the returned file path is none, the file doesn't exist
        if self.file_path is None:
            raise SettingNotFound("Settings.yml file not found")
        self.yaml_config = self._get_yaml_config()
        self.settings = self._get_settings()

    def _get_yaml_config(self):
        """
        Class Methods which loads default or the provided YAML file and return it as a String

        :return: The loaded settings YAML
        :rtype: dict

        :Example:
            settings_yaml = SettingLoader.get_yaml_config(/var/tmp/settings.yml)

        .. warnings:: Class Method
        """
        return YAMLLoader.get_config(self.file_path)

    def _get_settings(self):
        """
        Class Methods which loads default or the provided YAML file and return a Settings Object

        :return: The loaded Settings
        :rtype: Settings

        :Example:

            settings = SettingLoader.get_settings(file_path="/var/tmp/settings.yml")

        .. seealso:: Settings
        .. warnings:: Class Method
        """

        # create a new setting
        setting_object = Settings()

        # Get the setting parameters
        settings = self._get_yaml_config()
        default_stt_name = self._get_default_speech_to_text(settings)
        default_tts_name = self._get_default_text_to_speech(settings)
        default_trigger_name = self._get_default_trigger(settings)
        default_player_name = self._get_default_player(settings)
        stts = self._get_stts(settings)
        ttss = self._get_ttss(settings)
        triggers = self._get_triggers(settings)
        players = self._get_players(settings)
        rest_api = self._get_rest_api(settings)
        cache_path = self._get_cache_path(settings)
        resources = self._get_resources(settings)
        variables = self._get_variables(settings)
        recognition_options = self._get_recognition_options(settings)
        start_options = self._get_start_options(settings)
        hooks = self._get_hooks(settings)

        # Load the setting singleton with the parameters
        setting_object.default_tts_name = default_tts_name
        setting_object.default_stt_name = default_stt_name
        setting_object.default_trigger_name = default_trigger_name
        setting_object.default_player_name = default_player_name
        setting_object.stts = stts
        setting_object.ttss = ttss
        setting_object.triggers = triggers
        setting_object.players = players
        setting_object.rest_api = rest_api
        setting_object.cache_path = cache_path
        setting_object.resources = resources
        setting_object.variables = variables
        setting_object.recognition_options = recognition_options
        setting_object.start_options = start_options
        setting_object.hooks = hooks

        return setting_object

    @staticmethod
    def _get_default_speech_to_text(settings):
        """
        Get the default speech to text defined in the settings.yml file

        :param settings: The YAML settings file
        :type settings: dict
        :return: the default speech to text
        :rtype: str

        :Example:

            default_stt_name = cls._get_default_speech_to_text(settings)

        .. seealso:: Stt
        .. raises:: NullSettingException, SettingNotFound
        .. warnings:: Static and Private
        """

        try:
            default_speech_to_text = settings["default_speech_to_text"]
            if default_speech_to_text is None:
                raise NullSettingException("Attribute default_speech_to_text is null")
            logger.debug("Default STT: %s" % default_speech_to_text)
            return default_speech_to_text
        except KeyError as e:
            raise SettingNotFound("%s setting not found" % e)

    @staticmethod
    def _get_default_text_to_speech(settings):
        """
        Get the default text to speech defined in the settings.yml file

        :param settings: The YAML settings file
        :type settings: dict
        :return: the default text to speech
        :rtype: str

        :Example:

            default_tts_name = cls._get_default_text_to_speech(settings)

        .. seealso:: Tts
        .. raises:: NullSettingException, SettingNotFound
        .. warnings:: Static and Private
        """

        try:
            default_text_to_speech = settings["default_text_to_speech"]
            if default_text_to_speech is None:
                raise NullSettingException("Attribute default_text_to_speech is null")
            logger.debug("Default TTS: %s" % default_text_to_speech)
            return default_text_to_speech
        except KeyError as e:
            raise SettingNotFound("%s setting not found" % e)

    @staticmethod
    def _get_default_trigger(settings):
        """
        Get the default trigger defined in the settings.yml file
        :param settings: The YAML settings file
        :type settings: dict
        :return: the default trigger
        :rtype: str

        :Example:

            default_trigger_name = cls._get_default_trigger(settings)

        .. seealso:: Trigger
        .. raises:: NullSettingException, SettingNotFound
        .. warnings:: Static and Private
        """

        try:
            default_trigger = settings["default_trigger"]
            if default_trigger is None:
                raise NullSettingException("Attribute default_trigger is null")
            logger.debug("Default Trigger name: %s" % default_trigger)
            return default_trigger
        except KeyError as e:
            raise SettingNotFound("%s setting not found" % e)

    @staticmethod
    def _get_default_player(settings):
        """
        Get the default player defined in the settings.yml file
        :param settings: The YAML settings file
        :type settings: dict
        :return: the default player
        :rtype: str

        :Example:

            default_player_name = cls._get_default_player(settings)

        .. seealso:: Player
        .. raises:: NullSettingException, SettingNotFound
        .. warnings:: Static and Private
        """

        try:
            default_player = settings["default_player"]
            if default_player is None:
                raise NullSettingException("Attribute default_player is null")
            logger.debug("Default Player name: %s" % default_player)
            return default_player
        except KeyError as e:
            raise SettingNotFound("%s setting not found" % e)

    @staticmethod
    def _get_stts(settings):
        """
        Return a list of stt object

        :param settings: The YAML settings file
        :type settings: dict
        :return: List of Stt
        :rtype: list

        :Example:

            stts = cls._get_stts(settings)

        .. seealso:: Stt
        .. raises:: SettingNotFound
        .. warnings:: Static Method and Private
        """

        try:
            speechs_to_text_list = settings["speech_to_text"]
        except KeyError:
            raise SettingNotFound("speech_to_text settings not found")

        stts = list()
        for speechs_to_text_el in speechs_to_text_list:
            if isinstance(speechs_to_text_el, dict):
                for stt_name in speechs_to_text_el:
                    name = stt_name
                    parameters = speechs_to_text_el[name]
                    new_stt = Stt(name=name, parameters=parameters)
                    stts.append(new_stt)
            else:
                # the stt does not have parameter
                new_stt = Stt(name=speechs_to_text_el, parameters=dict())
                stts.append(new_stt)
        return stts

    @staticmethod
    def _get_ttss(settings):
        """

        Return a list of stt object

        :param settings: The YAML settings file
        :type settings: dict
        :return: List of Ttss
        :rtype: list

        :Example:

            ttss = cls._get_ttss(settings)

        .. seealso:: Tts
        .. raises:: SettingNotFound
        .. warnings:: Static Method and Private
        """

        try:
            text_to_speech_list = settings["text_to_speech"]
        except KeyError as e:
            raise SettingNotFound("%s setting not found" % e)

        ttss = list()
        for text_to_speech_el in text_to_speech_list:
            if isinstance(text_to_speech_el, dict):
                for tts_name in text_to_speech_el:
                    name = tts_name
                    parameters = text_to_speech_el[name]
                    new_tts = Tts(name=name, parameters=parameters)
                    ttss.append(new_tts)
            else:
                # the neuron does not have parameter
                new_tts = Tts(name=text_to_speech_el)
                ttss.append(new_tts)
        return ttss

    @staticmethod
    def _get_triggers(settings):
        """
        Return a list of Trigger object

        :param settings: The YAML settings file
        :type settings: dict
        :return: List of Trigger
        :rtype: list

        :Example:

            triggers = cls._get_triggers(settings)

        .. seealso:: Trigger
        .. raises:: SettingNotFound
        .. warnings:: Static Method and Private
        """

        try:
            triggers_list = settings["triggers"]
        except KeyError as e:
            raise SettingNotFound("%s setting not found" % e)

        triggers = list()
        for trigger_el in triggers_list:
            if isinstance(trigger_el, dict):
                for trigger_name in trigger_el:
                    name = trigger_name
                    parameters = trigger_el[name]
                    new_trigger = Trigger(name=name, parameters=parameters)
                    triggers.append(new_trigger)
            else:
                # the neuron does not have parameter
                new_trigger = Trigger(name=trigger_el)
                triggers.append(new_trigger)
        return triggers

    @staticmethod
    def _get_players(settings):
        """
        Return a list of Player object

        :param settings: The YAML settings file
        :type settings: dict
        :return: List of Player
        :rtype: list

        :Example:

            players = cls._get_players(settings)

        .. seealso:: players
        .. raises:: SettingNotFound
        .. warnings:: Static Method and Private
        """

        try:
            players_list = settings["players"]
        except KeyError as e:
            raise SettingNotFound("%s setting not found" % e)

        players = list()
        for player_el in players_list:
            if isinstance(player_el, dict):
                for player_name in player_el:
                    name = player_name
                    parameters = player_el[name]
                    new_player = Player(name=name, parameters=parameters)
                    players.append(new_player)
            else:
                # the player does not have parameters
                new_player = Player(name=player_el)
                players.append(new_player)
        return players

    @staticmethod
    def _get_rest_api(settings):
        """
        Return the settings of the RestApi

        :param settings: The YAML settings file
        :type settings: dict
        :return: the RestApi object
        :rtype: RestApi

        :Example:

            rest_api = cls._get_rest_api(settings)

        .. seealso:: RestApi
        .. raises:: SettingNotFound, NullSettingException, SettingInvalidException
        .. warnings:: Class Method and Private
        """

        try:
            rest_api = settings["rest_api"]
        except KeyError as e:
            raise SettingNotFound("%s setting not found" % e)

        if rest_api is not None:
            try:
                password_protected = rest_api["password_protected"]
                if password_protected is None:
                    raise NullSettingException("password_protected setting cannot be null")
                login = rest_api["login"]
                password = rest_api["password"]
                if password_protected:
                    if login is None:
                        raise NullSettingException("login setting cannot be null if password_protected is True")
                    if password is None:
                        raise NullSettingException("password setting cannot be null if password_protected is True")
                active = rest_api["active"]
                if active is None:
                    raise NullSettingException("active setting cannot be null")
                port = rest_api["port"]
                if port is None:
                    raise NullSettingException("port setting cannot be null")
                # check that the port in an integer
                try:
                    port = int(port)
                except ValueError:
                    raise SettingInvalidException("port must be an integer")
                # check the port is a valid port number
                if not 1024 <= port <= 65535:
                    raise SettingInvalidException("port must be in range 1024-65535")

                # check the CORS request settings
                allowed_cors_origin = False
                if "allowed_cors_origin" in rest_api:
                    allowed_cors_origin = rest_api["allowed_cors_origin"]

            except KeyError as e:
                raise SettingNotFound("%s settings not found" % e)

            # config ok, we can return the rest api object
            rest_api_obj = RestAPI(password_protected=password_protected, login=login, password=password,
                                   active=active, port=port, allowed_cors_origin=allowed_cors_origin)
            return rest_api_obj
        else:
            raise NullSettingException("rest_api settings cannot be null")

    @staticmethod
    def _get_cache_path(settings):
        """
        Return the path where to store the cache

        :param settings: The YAML settings file
        :type settings: dict
        :return: the path to store the cache
        :rtype: String

        :Example:

            cache_path = cls._get_cache_path(settings)

        .. seealso::
        .. raises:: SettingNotFound, NullSettingException, SettingInvalidException
        .. warnings:: Class Method and Private
        """

        try:
            cache_path = settings["cache_path"]
        except KeyError as e:
            raise SettingNotFound("%s setting not found" % e)

        if cache_path is None:
            raise NullSettingException("cache_path setting cannot be null")

        # test if that path is usable
        if FileManager.is_path_exists_or_creatable(cache_path):
            return cache_path
        else:
            raise SettingInvalidException("The cache_path seems to be invalid: %s" % cache_path)

    @staticmethod
    def _get_resources(settings):
        """
        Return a resources object that contains path of third party modules

        :param settings: The YAML settings file
        :type settings: dict
        :return: the resource object
        :rtype: Resources

        :Example:

            resource_directory = cls._get_resource_dir(settings)

        .. seealso::
        .. raises:: SettingNotFound, NullSettingException, SettingInvalidException
        .. warnings:: Class Method and Private
        """
        # return an empty resource object anyway
        resource_object = Resources()
        try:
            resource_dir = settings["resource_directory"]
            logger.debug("Resource directory synapse: %s" % resource_dir)

            neuron_folder = None
            stt_folder = None
            tts_folder = None
            trigger_folder = None
            signal_folder = None

            if "neuron" in resource_dir:
                neuron_folder = resource_dir["neuron"]
                if os.path.exists(neuron_folder):
                    logger.debug("[SettingLoader] Neuron resource folder path loaded: %s" % neuron_folder)
                    resource_object.neuron_folder = neuron_folder
                else:
                    raise SettingInvalidException("The path %s does not exist on the system" % neuron_folder)

            if "stt" in resource_dir:
                stt_folder = resource_dir["stt"]
                if os.path.exists(stt_folder):
                    logger.debug("[SettingLoader] STT resource folder path loaded: %s" % stt_folder)
                    resource_object.stt_folder = stt_folder
                else:
                    raise SettingInvalidException("The path %s does not exist on the system" % stt_folder)

            if "tts" in resource_dir:
                tts_folder = resource_dir["tts"]
                if os.path.exists(tts_folder):
                    logger.debug("[SettingLoader] TTS resource folder path loaded: %s" % tts_folder)
                    resource_object.tts_folder = tts_folder
                else:
                    raise SettingInvalidException("The path %s does not exist on the system" % tts_folder)

            if "trigger" in resource_dir:
                trigger_folder = resource_dir["trigger"]
                if os.path.exists(trigger_folder):
                    logger.debug("[SettingLoader] Trigger resource folder path loaded: %s" % trigger_folder)
                    resource_object.trigger_folder = trigger_folder
                else:
                    raise SettingInvalidException("The path %s does not exist on the system" % trigger_folder)

            if "signal" in resource_dir:
                signal_folder = resource_dir["signal"]
                if os.path.exists(signal_folder):
                    logger.debug("[SettingLoader] Signal resource folder path loaded: %s" % signal_folder)
                    resource_object.signal_folder = signal_folder
                else:
                    raise SettingInvalidException("The path %s does not exist on the system" % signal_folder)

            if neuron_folder is None \
                    and stt_folder is None \
                    and tts_folder is None \
                    and trigger_folder is None \
                    and signal_folder is None:
                raise SettingInvalidException("No required folder has been provided in the setting resource_directory. "
                                              "Define : \'neuron\' or/and \'stt\' or/and \'tts\' or/and \'trigger\' "
                                              "or/and \'signal\'")

        except KeyError:
            logger.debug("Resource directory not found in settings")
            return resource_object

        return resource_object

    @staticmethod
    def _get_variables(settings):
        """
        Return the dict of variables from the settings.
        :param settings: The YAML settings file
        :return: dict
        """

        variables = dict()
        try:
            variables_files_name = settings["var_files"]
            # In case files are declared in settings.yml, make sure kalliope can access them.
            for files in variables_files_name:
                var = Utils.get_real_file_path(files)
                if var is None:
                    raise SettingInvalidException("Variables file %s not found" % files)
                else:
                    variables.update(YAMLLoader.get_config(var))
            return variables
        except KeyError:
            # User does not provide this settings
            return dict()

    @staticmethod
    def _get_recognition_options(settings):
        """
        return the value of stt_threshold
        :param settings: The loaded YAML settings file
        :return: integer or 1200 by default if not set
        """
        recognition_options = RecognitionOptions()

        try:
            recognition_options_dict = settings["recognition_options"]

            if "energy_threshold" in recognition_options_dict:
                recognition_options.energy_threshold = recognition_options_dict["energy_threshold"]
                logger.debug("[SettingsLoader] energy_threshold set to %s" % recognition_options.energy_threshold)
            if "adjust_for_ambient_noise_second" in recognition_options_dict:
                recognition_options.adjust_for_ambient_noise_second = recognition_options_dict["adjust_for_ambient_noise_second"]
                logger.debug("[SettingsLoader] adjust_for_ambient_noise_second set to %s"
                             % recognition_options.adjust_for_ambient_noise_second)
            return recognition_options

        except KeyError:
            logger.debug("[SettingsLoader] no recognition_options defined. Set to default")

        logger.debug("[SettingsLoader] recognition_options: %s" % str(recognition_options))
        return recognition_options

    @staticmethod
    def _get_start_options(settings):
        """
        Return the start options settings

        :param settings: The YAML settings file
        :type settings: dict
        :return: A dict containing the start options
        :rtype: dict
        """
        options = dict()
        muted = False

        try:
            start_options = settings["start_options"]
        except KeyError:
            start_options = None

        if start_options is not None:
            try:
                muted = start_options['muted']
            except KeyError:
                muted = False

        options['muted'] = muted

        logger.debug("Start options: %s" % options)
        return options

    @staticmethod
    def _get_hooks(settings):
        """
        Return hooks settings
        :param settings: The YAML settings file
        :return: A dict containing hooks
        :rtype: dict
        """

        try:
            hooks = settings["hooks"]

        except KeyError:
            # if the user haven't set any hooks we define an empty dict
            hooks = dict()

        all_hook = [
            "on_start",
            "on_waiting_for_trigger",
            "on_triggered",
            "on_start_listening",
            "on_stop_listening",
            "on_order_found",
            "on_order_not_found",
            "on_mute",
            "on_unmute",
            "on_start_speaking",
            "on_stop_speaking"
        ]

        for key in all_hook:
            if key not in hooks:
                hooks[key] = None

        return hooks



import logging
import os
import inspect
import imp
import sys
import re
import six

logging.basicConfig()
logger = logging.getLogger("kalliope")


def pipe_print(line):
    line = Utils.encode_text_utf8(line)
    print(line)


class ModuleNotFoundError(Exception):
    """
    The module can not been found

    .. notes: Check the case: must be in lower case.
    """
    pass


class Utils(object):
    color_list = dict(
        PURPLE='\033[95m',
        BLUE='\033[94m',
        GREEN='\033[92m',
        YELLOW='\033[93m',
        RED='\033[91m',
        ENDLINE='\033[0m',
        BOLD='\033[1m',
        UNDERLINE='\033[4m'
    )

    ##################
    #
    # Shell properly displayed
    #
    #########
    @classmethod
    def print_info(cls, text_to_print):
        pipe_print(cls.color_list["BLUE"] + text_to_print + cls.color_list["ENDLINE"])
        logger.debug(text_to_print)

    @classmethod
    def print_success(cls, text_to_print):
        pipe_print(cls.color_list["GREEN"] + text_to_print + cls.color_list["ENDLINE"])
        logger.debug(text_to_print)

    @classmethod
    def print_warning(cls, text_to_print):
        pipe_print(cls.color_list["YELLOW"] + text_to_print + cls.color_list["ENDLINE"])
        logger.debug(text_to_print)

    @classmethod
    def print_danger(cls, text_to_print):
        pipe_print(cls.color_list["RED"] + text_to_print + cls.color_list["ENDLINE"])
        logger.debug(text_to_print)

    @classmethod
    def print_header(cls, text_to_print):
        pipe_print(cls.color_list["HEADER"] + text_to_print + cls.color_list["ENDLINE"])
        logger.debug(text_to_print)

    @classmethod
    def print_purple(cls, text_to_print):
        pipe_print(cls.color_list["PURPLE"] + text_to_print + cls.color_list["ENDLINE"])
        logger.debug(text_to_print)

    @classmethod
    def print_bold(cls, text_to_print):
        pipe_print(cls.color_list["BOLD"] + text_to_print + cls.color_list["ENDLINE"])
        logger.debug(text_to_print)

    @classmethod
    def print_underline(cls, text_to_print):
        pipe_print(cls.color_list["UNDERLINE"] + text_to_print + cls.color_list["ENDLINE"])
        logger.debug(text_to_print)

    @staticmethod
    def print_yaml_nicely(to_print):
        """
        Used for debug
        :param to_print: Dict to print nicely
        :return:
        """
        import json
        line = json.dumps(to_print, indent=2)
        return line.encode('utf-8')

    ##################
    #
    # Dynamic loading
    #
    #########
    @classmethod
    def get_dynamic_class_instantiation(cls, package_name, module_name, parameters=None, resources_dir=None):
        """
        Load a python class dynamically

        from my_package.my_module import my_class
        mod = __import__('my_package.my_module', fromlist=['my_class'])
        klass = getattr(mod, 'my_class')

        :param package_name: name of the package where we will find the module to load (neurons, tts, stt, trigger)
        :param module_name: name of the module from the package_name to load. This one is capitalized. Eg: Snowboy
        :param parameters:  dict parameters to send as argument to the module
        :param resources_dir: the resource directory to check for external resources
        :return:
        """
        package_path = "kalliope." + package_name + "." + module_name.lower() + "." + module_name.lower()
        logger.debug("[Utils]-> get_dynamic_class_instantiation : package path : %s" % (package_path))
        if resources_dir is not None:
            neuron_resource_path = resources_dir + os.sep + module_name.lower() \
                                   + os.sep + module_name.lower() + ".py"
            if os.path.exists(neuron_resource_path):
                imp.load_source(module_name.capitalize(), neuron_resource_path)
                package_path = module_name.capitalize()
                logger.debug("[Utils]-> get_dynamic_class_instantiation : loading path : %s, as package %s" % (
                    neuron_resource_path, package_path))

        mod = __import__(package_path, fromlist=[module_name.capitalize()])

        try:
            klass = getattr(mod, module_name.capitalize())
        except AttributeError:
            logger.debug("Error: No module named %s " % module_name.capitalize())
            raise ModuleNotFoundError(
                "The module %s does not exist in package %s" % (module_name.capitalize(), package_name))

        if klass is not None:
            # run the plugin
            if not parameters:
                return klass()
            elif isinstance(parameters, dict):
                return klass(**parameters)
            else:
                return klass(parameters)
        return None

    ##################
    #
    # Paths management
    #
    #########
    @staticmethod
    def get_current_file_parent_parent_path(current_script_path):
        parent_parent_path = os.path.normpath(current_script_path + os.sep + os.pardir + os.sep + os.pardir)
        return parent_parent_path

    @staticmethod
    def get_current_file_parent_path(current_script_path):
        parent_path = os.path.normpath(current_script_path + os.sep + os.pardir)
        return parent_path

    @classmethod
    def get_real_file_path(cls, file_path_to_test):
        """
        Try to return a full path from a given <file_path_to_test>
        If the path is an absolute on, we return it directly.

        If the path is relative, we try to get the full path in this order:
        - from the current directory where kalliope has been called + the file_path_to_test.
        Eg: /home/me/Documents/kalliope_config
        - from /etc/kalliope + file_path_to_test
        - from the default file passed as <file_name> at the root of the project

        :param file_path_to_test file path to test
        :type file_path_to_test: str
        :return: absolute path to the file file_path_to_test or None if is doen't exist
        """

        if not os.path.isabs(file_path_to_test):
            current_script_path = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
            path_order = {
                1: os.getcwd() + os.sep + file_path_to_test,
                2: "/etc/kalliope" + os.sep + file_path_to_test,
                # In this case 'get_current_file_parent_parent_path' is corresponding to kalliope root path
                # from /an/unknown/path/kalliope/kalliope/core/Utils to /an/unknown/path/kalliope/kalliope
                3: cls.get_current_file_parent_parent_path(current_script_path) + os.sep + file_path_to_test
            }

            for key in sorted(path_order):
                new_file_path_to_test = path_order[key]
                logger.debug("Try to load file from %s: %s" % (key, new_file_path_to_test))
                if os.path.isfile(new_file_path_to_test):
                    logger.debug("File found in %s" % new_file_path_to_test)
                    return new_file_path_to_test

        else:
            if os.path.isfile(file_path_to_test):
                return file_path_to_test
            else:
                return None

    @staticmethod
    def query_yes_no(question, default="yes"):
        """Ask a yes/no question via raw_input() and return their answer.

        "question" is a string that is presented to the user.
        "default" is the presumed answer if the user just hits <Enter>.
            It must be "yes" (the default), "no" or None (meaning
            an answer is required of the user).

        The "answer" return value is True for "yes" or False for "no".
        """
        valid = {"yes": True, "y": True, "ye": True,
                 "no": False, "n": False}
        if default is None:
            prompt = " [y/n] "
        elif default == "yes":
            prompt = " [Y/n] "
        elif default == "no":
            prompt = " [y/N] "
        else:
            raise ValueError("invalid default answer: '%s'" % default)

        while True:
            Utils.print_warning(question + prompt)
            choice = raw_input().lower()
            if default is not None and choice == '':
                return valid[default]
            elif choice in valid:
                return valid[choice]
            else:
                Utils.print_warning("Please respond with 'yes' or 'no' or 'y' or 'n').\n")

    ##################
    #
    # Brackets management
    #
    #########
    @staticmethod
    def is_containing_bracket(sentence):
        """
        Return True if the text in <sentence> contains brackets
        :param sentence:
        :return:
        """
        # print "sentence to test %s" % sentence
        pattern = r"{{|}}"
        # prog = re.compile(pattern)
        if not isinstance(sentence, six.text_type):
            sentence = str(sentence)
        check_bool = re.search(pattern, sentence)
        if check_bool is not None:
            return True
        return False

    @staticmethod
    def find_all_matching_brackets(sentence):
        """
        Find all the bracket matches from a given sentence
        :param sentence: the sentence to check
        :return: the list with all the matches
        """

        pattern = r"((?:{{\s*)[\w\.]+(?:\s*}}))"
        # find everything like {{ word }}
        if not isinstance(sentence, six.text_type):
            sentence = str(sentence)
        return re.findall(pattern, sentence)

    @staticmethod
    def remove_spaces_in_brackets(sentence):
        """
        If has brackets it removes spaces in brackets
        :param sentence: the sentence to work on
        :return: the sentence without any spaces in brackets
        """

        pattern = '(?<=\{\{)\s+|\s+(?=\}\})'
        # Remove white spaces (if any) between the variable and the double brace then split
        if not isinstance(sentence, six.text_type):
            sentence = str(sentence)
        return re.sub(pattern, '', sentence)

    ##################
    #
    # Lists management
    #
    #########
    @staticmethod
    def get_next_value_list(list_to_check):
        ite = list_to_check.__iter__()
        next(ite, None)
        return next(ite, None)

    ##################
    #
    # Encoding
    #
    #########
    @staticmethod
    def encode_text_utf8(text):
        if sys.version_info[0] < 3:
            if isinstance(text, unicode):
                text = text.encode("utf-8")
        return text



import os
import unittest

from mock import mock

from kalliope import ResourcesManager
from kalliope.core.Models import Resources
from kalliope.core.Models.Dna import Dna


class TestResourcesmanager(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_is_settings_ok(self):
        # -----------------
        # valid resource
        # -----------------
        # valid neuron
        valid_resource = Resources()
        valid_resource.neuron_folder = "/path"
        dna = Dna()
        dna.module_type = "neuron"
        self.assertTrue(ResourcesManager.is_settings_ok(valid_resource, dna))

        # valid stt
        valid_resource = Resources()
        valid_resource.stt_folder = "/path"
        dna = Dna()
        dna.module_type = "stt"
        self.assertTrue(ResourcesManager.is_settings_ok(valid_resource, dna))

        # valid tts
        valid_resource = Resources()
        valid_resource.tts_folder = "/path"
        dna = Dna()
        dna.module_type = "tss"
        self.assertTrue(ResourcesManager.is_settings_ok(valid_resource, dna))

        # valid trigger
        valid_resource = Resources()
        valid_resource.trigger_folder = "/path"
        dna = Dna()
        dna.module_type = "trigger"
        self.assertTrue(ResourcesManager.is_settings_ok(valid_resource, dna))

        # valid signal
        valid_resource = Resources()
        valid_resource.signal_folder = "/path"
        dna = Dna()
        dna.module_type = "signal"
        self.assertTrue(ResourcesManager.is_settings_ok(valid_resource, dna))

        # -----------------
        # invalid resource
        # -----------------
        # valid neuron
        valid_resource = Resources()
        valid_resource.neuron_folder = None
        dna = Dna()
        dna.module_type = "neuron"
        self.assertFalse(ResourcesManager.is_settings_ok(valid_resource, dna))

        # valid stt
        valid_resource = Resources()
        valid_resource.stt_folder = None
        dna = Dna()
        dna.module_type = "stt"
        self.assertFalse(ResourcesManager.is_settings_ok(valid_resource, dna))

        # valid tts
        valid_resource = Resources()
        valid_resource.tts_folder = None
        dna = Dna()
        dna.module_type = "tts"
        self.assertFalse(ResourcesManager.is_settings_ok(valid_resource, dna))

        # valid trigger
        valid_resource = Resources()
        valid_resource.trigger_folder = None
        dna = Dna()
        dna.module_type = "trigger"
        self.assertFalse(ResourcesManager.is_settings_ok(valid_resource, dna))

        # valid signal
        valid_resource = Resources()
        valid_resource.signal_folder = None
        dna = Dna()
        dna.module_type = "signal"
        self.assertFalse(ResourcesManager.is_settings_ok(valid_resource, dna))

    def test_is_repo_ok(self):
        # valid repo
        if "/Tests" in os.getcwd():
            dna_file_path = "modules/dna.yml"
            install_file_path = "modules/install.yml"
        else:
            dna_file_path = "Tests/modules/dna.yml"
            install_file_path = "Tests/modules/install.yml"
        self.assertTrue(ResourcesManager.is_repo_ok(dna_file_path=dna_file_path, install_file_path=install_file_path))

        # missing dna
        if "/Tests" in os.getcwd():
            dna_file_path = ""
            install_file_path = "modules/install.yml"
        else:
            dna_file_path = "T"
            install_file_path = "Tests/modules/install.yml"
        self.assertFalse(ResourcesManager.is_repo_ok(dna_file_path=dna_file_path, install_file_path=install_file_path))

        # missing install
        if "/Tests" in os.getcwd():
            dna_file_path = "modules/dna.yml"
            install_file_path = ""
        else:
            dna_file_path = "Tests/modules/dna.yml"
            install_file_path = ""
        self.assertFalse(ResourcesManager.is_repo_ok(dna_file_path=dna_file_path, install_file_path=install_file_path))

    def test_get_target_folder(self):
        # test get neuron folder
        resources = Resources()
        resources.neuron_folder = '/var/tmp/test/resources'
        self.assertEqual(ResourcesManager._get_target_folder(resources, "neuron"), "/var/tmp/test/resources")

        # test get stt folder
        resources = Resources()
        resources.stt_folder = '/var/tmp/test/resources'
        self.assertEqual(ResourcesManager._get_target_folder(resources, "stt"), "/var/tmp/test/resources")

        # test get tts folder
        resources = Resources()
        resources.tts_folder = '/var/tmp/test/resources'
        self.assertEqual(ResourcesManager._get_target_folder(resources, "tts"), "/var/tmp/test/resources")

        # test get trigger folder
        resources = Resources()
        resources.trigger_folder = '/var/tmp/test/resources'
        self.assertEqual(ResourcesManager._get_target_folder(resources, "trigger"), "/var/tmp/test/resources")

        # test get signal folder
        resources = Resources()
        resources.signal_folder = '/var/tmp/test/resources'
        self.assertEqual(ResourcesManager._get_target_folder(resources, "signal"), "/var/tmp/test/resources")

        # test get non existing resource
        resources = Resources()
        self.assertIsNone(ResourcesManager._get_target_folder(resources, "not_existing"))

    def test_check_supported_version(self):
        # version ok
        current_version = '0.4.0'
        supported_version = ['0.4', '0.3', '0.2']

        self.assertTrue(ResourcesManager._check_supported_version(current_version=current_version,
                                                                  supported_versions=supported_version))

        # version ok
        current_version = '11.23.0'
        supported_version = ['11.23', '12.3', '2.23']

        self.assertTrue(ResourcesManager._check_supported_version(current_version=current_version,
                                                                  supported_versions=supported_version))

        # version non ok, user does not config
        # Testing with integer values instead of string
        current_version = '0.4.0'
        supported_version = [0.3, 0.2]

        with mock.patch('kalliope.Utils.query_yes_no', return_value=True):
            self.assertTrue(ResourcesManager._check_supported_version(current_version=current_version,
                                                                      supported_versions=supported_version))

        with mock.patch('kalliope.Utils.query_yes_no', return_value=False):
            self.assertFalse(ResourcesManager._check_supported_version(current_version=current_version,
                                                                       supported_versions=supported_version))



import logging

from kalliope import Utils
from kalliope.signals.order import Order

logging.basicConfig()
logger = logging.getLogger("kalliope")


class SignalLauncher:

    # keep a list of instantiated signals
    list_launched_signals = list()

    def __init__(self):
        pass

    @classmethod
    def launch_signal_class_by_name(cls, signal_name, settings=None):
        """
        load the signal class from the given name, pass the brain and settings to the signal
        :param signal_name: name of the signal class to load
        :param settings: Settings Object
        """
        signal_folder = None
        if settings.resources:
            signal_folder = settings.resources.signal_folder

        launched_signal = Utils.get_dynamic_class_instantiation(package_name="signals",
                                                                module_name=signal_name,
                                                                resources_dir=signal_folder)

        cls.add_launched_signals_to_list(launched_signal)

        return launched_signal

    @classmethod
    def add_launched_signals_to_list(cls, signal):
        cls.list_launched_signals.append(signal)

    @classmethod
    def get_launched_signals_list(cls):
        return cls.list_launched_signals

    @classmethod
    def get_order_instance(cls):
        """
        Return the Order instance from the list of launched signals if exist
        :return:
        """
        for signal in cls.list_launched_signals:
            if isinstance(signal, Order):
                return signal
        return None



import copy

from kalliope.core.NeuronParameterLoader import NeuronParameterLoader


class MatchedSynapse(object):
    """
    This class represent a synapse that has matched an order send by an User.
    """

    def __init__(self, matched_synapse=None, matched_order=None, user_order=None, overriding_parameter=None):
        """
        :param matched_synapse: The synapse that has matched in the brain.
        :param matched_order: The order from the synapse that have matched.
        :param user_order: The order said by the user.
        :param overriding_parameter: If set, those parameters will over
        """

        # create a copy of the synapse. the received synapse come from the brain.
        self.synapse = matched_synapse
        # create a fifo list that contains all neurons to process.
        # Create a copy to be sure when we remove a neuron from this list it will not be removed from the synapse's
        # neuron list
        self.neuron_fifo_list = copy.deepcopy(self.synapse.neurons)
        self.matched_order = matched_order
        self.parameters = dict()
        if matched_order is not None:
            self.parameters = NeuronParameterLoader.get_parameters(synapse_order=self.matched_order,
                                                                   user_order=user_order)
        if overriding_parameter is not None:
            # merge dict of parameters with overriding
            self.parameters.update(overriding_parameter)

        # list of Neuron Module
        self.neuron_module_list = list()

    def __str__(self):
        return str(self.serialize())

    def serialize(self):
        """
        This method allows to serialize in a proper way this object

        :return: A dict of name and parameters
        :rtype: Dict
        """
        return {
            'synapse_name': self.synapse.name,
            'matched_order': self.matched_order,
            'neuron_module_list': [e.serialize() for e in self.neuron_module_list]
        }

    def __eq__(self, other):
        """
        This is used to compare 2 objects
        :param other:
        :return:
        """
        return self.__dict__ == other.__dict__



# -*- coding: utf-8 -*-
import logging

import sounddevice as sd
import soundfile as sf

from kalliope.core.PlayerModule import PlayerModule

logging.basicConfig()
logger = logging.getLogger("kalliope")

FS = 48000


class Sounddeviceplayer(PlayerModule):
    """
    This Class is representing the Player Object used to play the all sound of the system.
    """

    def __init__(self, **kwargs):
        super(Sounddeviceplayer, self).__init__(**kwargs)
        logger.debug("[Sounddeviceplayer.__init__] instance")
        logger.debug("[Sounddeviceplayer.__init__] args : %s " % str(kwargs))

    def play(self, file_path):

        if self.convert:
            self.convert_mp3_to_wav(file_path_mp3=file_path)
        data, fs = sf.read(file_path)
        sd.play(data, fs)
        sd.wait()



import json
import logging
import os

import requests

from kalliope.core.NeuronModule import NeuronModule, InvalidParameterException

logging.basicConfig()
logger = logging.getLogger("kalliope")


class Uri(NeuronModule):

    def __init__(self, **kwargs):
        super(Uri, self).__init__(**kwargs)

        # input variables
        self.url = kwargs.get('url', None)
        self.headers = kwargs.get('headers', None)
        self.data = kwargs.get('data', None)
        self.data_from_file = kwargs.get('data_from_file', None)
        self.method = kwargs.get('method', "GET")
        self.user = kwargs.get('user', None)
        self.password = kwargs.get('password', None)
        self.timeout = kwargs.get('timeout', None)

        # processing parameters
        self.parameters = None

        # output variable
        self.status_code = None
        self.content = None
        self.text = None
        self.response_header = None

        # this is a switch case option
        switch_case = {
            "GET": self.do_get,
            "POST": self.do_post,
            "DELETE": self.do_delete,
            "PUT": self.do_put,
            "HEAD": self.do_head,
            "PATCH": self.do_patch,
            "OPTIONS": self.do_options
        }

        # check parameters
        if self._is_parameters_ok():
            # we get parameters that will be passed to the requests lib
            self.parameters = self.get_parameters()
            # we call the right method depending of the method selected
            switch_case[self.method]()

            message = {
                "status_code": self.status_code,
                "content": self.content,
                "response_header": self.response_header
            }

            self.say(message)

    def do_get(self):
        logger.debug(self.neuron_name + " do_get method called")
        r = requests.get(url=self.url, **self.parameters)
        self.post_processing_request(r)

    def do_post(self):
        logger.debug(self.neuron_name + " do_post method called")
        r = requests.post(url=self.url, **self.parameters)
        self.post_processing_request(r)

    def do_delete(self):
        logger.debug(self.neuron_name + " do_delete method called")
        r = requests.delete(url=self.url, **self.parameters)
        self.post_processing_request(r)

    def do_put(self):
        logger.debug(self.neuron_name + " do_put method called")
        r = requests.put(url=self.url, **self.parameters)
        self.post_processing_request(r)

    def do_head(self):
        logger.debug(self.neuron_name + " do_head method called")
        r = requests.head(url=self.url, **self.parameters)
        self.post_processing_request(r)

    def do_patch(self):
        logger.debug(self.neuron_name + " do_patch method called")
        r = requests.patch(url=self.url, **self.parameters)
        self.post_processing_request(r)

    def do_options(self):
        logger.debug(self.neuron_name + " do_options method called")
        r = requests.options(url=self.url, **self.parameters)
        self.post_processing_request(r)

    def get_parameters(self):
        """

        :return: Dict of parameters usable by the "requests" lib
        """
        returned_parameters = dict()

        if self.headers is not None:
            returned_parameters["headers"] = self.headers

        if self.timeout is not None:
            returned_parameters["timeout"] = self.timeout

        if self.data is not None:
            returned_parameters["data"] = self.data

        if self.data_from_file is not None:
            returned_parameters["data"] = self.data_from_file

        if self.user is not None:
            # this implicitly means that the password is set too, the check has been done in _is_parameters_ok
            returned_parameters["auth"] = self.user, self.password

        logger.debug(self.neuron_name + " parameters: %s" % returned_parameters)

        return returned_parameters

    def post_processing_request(self, r):
        self.status_code = r.status_code
        self.content = r.content
        # we try to load into a json object the content. So Kalliope can use it to talk
        try:
            self.content = json.loads(self.content.decode())
        except ValueError:
            logger.debug(self.neuron_name + "cannot get a valid json from returned content")
            pass
        self.text = r.text
        self.response_header = r.headers

        logger.debug(self.neuron_name + " status_code: %s" % self.status_code)
        logger.debug(self.neuron_name + " content: %s" % self.content)
        logger.debug(self.neuron_name + " response_header: %s" % self.response_header)

    def _is_parameters_ok(self):
        """
        Check that all provided parameters in the neurons are valid
        :return: True if all check passed
        """
        # URL is mandatory
        if self.url is None:
            raise InvalidParameterException("Uri needs an url")

        # headers can be null, but if provided it must be a list
        if self.headers is not None:
            if not isinstance(self.headers, dict):
                raise InvalidParameterException("headers must be a list of key: value")

        # timeout in second must be an integer
        if self.timeout is not None:
            if not isinstance(self.timeout, int):
                raise InvalidParameterException("timeout must be an integer")

        # data must be loadable with json
        if self.data is not None:
            try:
                json.loads(self.data)
            except ValueError as e:
                raise InvalidParameterException("error in \"data\" parameter: %s" % e)

        # data_from_file path must exist and data inside must be loadable by json
        if self.data_from_file is not None:
            # check that the file exist
            if not os.path.exists(self.data_from_file):
                raise InvalidParameterException("error in \"data_file\". File does not exist: %s" % self.data_from_file)
            # then try to load the json from the file
            try:
                self.data_from_file = self.readfile(self.data_from_file)
            except ValueError as e:
                raise InvalidParameterException("error in \"data\" parameter: %s" % e)

        # we cannot provide both data and data from file
        if self.data is not None and self.data_from_file is not None:
            raise InvalidParameterException("URI can be used with data or data_from_file, not both in same time")

        # the provided method must exist
        allowed_method = ["GET", "POST", "DELETE", "PUT", "HEAD", "PATCH", "OPTIONS"]
        if self.method not in allowed_method:
            raise InvalidParameterException("method %s not in: %s" % (self.method, allowed_method))

        return True

    @staticmethod
    def readfile(file_path):
        """
        return the content of the file <file_path>
        :param file_path: File path to read
        :return: Str content of the file
        """
        file_to_read = open(file_path, 'r')
        return file_to_read.read()



import logging

import jinja2
from kalliope.core.Utils.Utils import Utils

from kalliope.core.Models import Singleton
from six import with_metaclass

logging.basicConfig()
logger = logging.getLogger("kalliope")


class Cortex(with_metaclass(Singleton, object)):
    """
    short-term memories of kalliope. Used to store object with a "key" "value"
    """
    # this dict contains the short term memory of kalliope.
    # all keys present in this dict has been saved from a user demand
    memory = dict()
    # this is a temp dict that allow us to store temporary parameters that as been loaded from the user order
    # if the user want to a key from this dict, the key and its value will be added o the memory dict
    temp = dict()

    def __init__(self):
        logger.debug("[Cortex] New memory created")

    @classmethod
    def get_memory(cls):
        """
        Get the current dict of parameters saved in memory
        :return: dict memory
        """
        return cls.memory

    @classmethod
    def save(cls, key, value):
        """
        Save a new value in the memory
        :param key: key to save
        :param value: value to save into the key
        """
        if key in cls.memory:
            logger.debug("[Cortex] key %s already present in memory with value %s. Will be overridden"
                         % (key, cls.memory[key]))
        logger.debug("[Cortex] key saved in memory. key: %s, value: %s" % (key, value))
        cls.memory[key] = value

    @classmethod
    def get_from_key(cls, key):
        try:
            return cls.memory[key]
        except KeyError:
            logger.debug("[Cortex] key %s does not exist in memory" % key)
            return None

    @classmethod
    def add_parameters_from_order(cls, dict_parameter):
        logger.debug("[Cortex] place parameters in temp list: %s" % dict_parameter)
        cls.temp.update(dict_parameter)

    @classmethod
    def clean_parameter_from_order(cls):
        """
        Clean the temps memory that store parameters loaded from vocal order
        """
        logger.debug("[Cortex] Clean temp memory")
        cls.temp = dict()

    @classmethod
    def save_neuron_parameter_in_memory(cls, kalliope_memory_dict, neuron_parameters):
        """
        receive a dict of value send by the child neuron
        save in kalliope memory all value

        E.g
        dict_parameter_to_save = {"my_key_to_save_in_memory": "{{ output_val_from_neuron }}"}
        neuron_parameter = {"output_val_from_neuron": "this_is_a_value" }

        then the cortex will save in memory the key "my_key_to_save_in_memory" and attach the value "this_is_a_value"

        :param neuron_parameters: dict of parameter the neuron has processed and send to the neurone module to
                be processed by the TTS engine
        :param kalliope_memory_dict: a dict of key value the user want to save from the dict_neuron_parameter
        """

        if kalliope_memory_dict is not None:
            logger.debug("[Cortex] save_memory - User want to save: %s" % kalliope_memory_dict)
            logger.debug("[Cortex] save_memory - Available parameters in the neuron: %s" % neuron_parameters)

            for key, value in kalliope_memory_dict.items():
                # ask the cortex to save in memory the target "key" if it was in parameters of the neuron
                if isinstance(neuron_parameters, dict):
                    if Utils.is_containing_bracket(value):
                        value = jinja2.Template(value).render(neuron_parameters)
                    Cortex.save(key, value)

    @classmethod
    def save_parameter_from_order_in_memory(cls, order_parameters):
        """
        Save key from the temp dict (where parameters loaded from the voice order where placed temporary)
        into the memory dict
        :param order_parameters: dict of key to save.  {'key_name_in_memory': 'key_name_in_temp_dict'}
        :return True if a value has been saved in the kalliope memory
        """
        order_saved = False
        if order_parameters is not None:
            logger.debug("[Cortex] save_parameter_from_order_in_memory - User want to save: %s" % order_parameters)
            logger.debug("[Cortex] save_parameter_from_order_in_memory - Available parameters in orders: %s"
                         % cls.temp)

            for key, value in order_parameters.items():
                # ask the cortex to save in memory the target "key" if it was in the order
                if Utils.is_containing_bracket(value):
                    # if the key exist in the temp dict we can load it with jinja
                    value = jinja2.Template(value).render(Cortex.temp)
                    if value:
                        Cortex.save(key, value)
                        order_saved = True

        return order_saved



# def test_check_order_dict(self):
#     valid_order = 'test_order'
#     invalid_order = ''
#     invalid_order2 = None
#
#     self.assertTrue(ConfigurationChecker.check_order_dict(valid_order))
#
#     with self.assertRaises(NoValidOrder):
#         ConfigurationChecker.check_order_dict(invalid_order)
#     with self.assertRaises(NoValidOrder):
#         ConfigurationChecker.check_order_dict(invalid_order2)


import logging
from threading import Thread

from kalliope.core import SignalModule

logging.basicConfig()
logger = logging.getLogger("kalliope")


class Geolocation(SignalModule, Thread):
    def __init__(self, **kwargs):
        super(Geolocation, self).__init__(**kwargs)

    def run(self):
        logger.debug("[Geolocalisation] Loading ...")
        self.list_synapses_with_geolocalion = list(super(Geolocation, self).get_list_synapse())

    @staticmethod
    def check_parameters(parameters):
        """
        Overwritten method
        receive a dict of parameter from a geolocation signal and them
        :param parameters: dict of parameters
        :return: True if parameters are valid
        """
        # check mandatory parameters
        mandatory_parameters = ["latitude", "longitude", "radius"]
        return all(key in parameters for key in mandatory_parameters)



#
# def test_check_event_dict(self):
#     valid_event = {
#         "hour": "18",
#         "minute": "16"
#     }
#     invalid_event = None
#     invalid_event2 = ""
#     invalid_event3 = {
#         "notexisting": "12"
#     }
#
#     self.assertTrue(ConfigurationChecker.check_event_dict(valid_event))
#
#     with self.assertRaises(NoEventPeriod):
#         ConfigurationChecker.check_event_dict(invalid_event)
#     with self.assertRaises(NoEventPeriod):
#         ConfigurationChecker.check_event_dict(invalid_event2)
#     with self.assertRaises(NoEventPeriod):
#         ConfigurationChecker.check_event_dict(invalid_event3)


# coding: utf8
import logging
import os
import subprocess

from kalliope.core.Utils.FileManager import FileManager

logging.basicConfig()
logger = logging.getLogger("kalliope")


class PlayerModule(object):
    """
    Mother class of Players. 
    Ability to convert mp3 to wave format.
    """

    def __init__(self, **kwargs):

        # set parameter from what we receive from the settings
        self.convert = kwargs.get('convert_to_wav', True)

    @staticmethod
    def convert_mp3_to_wav(file_path_mp3):
        """ 
        PyAudio, AlsaPlayer, sounddevices  do not support mp3 files 
        MP3 files must be converted to a wave in order to be played
        This function assumes ffmpeg is available on the system
        :param file_path_mp3: the file path to convert from mp3 to wav
        """
        logger.debug("Converting mp3 file to wav file: %s" % file_path_mp3)
        fnull = open(os.devnull, 'w')
        # temp file
        tmp_file_wav = file_path_mp3 + ".wav"
        # Convert mp3 to wave
        subprocess.call(['avconv', '-y', '-i', file_path_mp3, tmp_file_wav],
                        stdout=fnull, stderr=fnull)
        # remove the original file
        FileManager.remove_file(file_path_mp3)
        # rename the temp file with the same name as the original file
        os.rename(tmp_file_wav, file_path_mp3)


import unittest
import mock

from kalliope.core.Models.Player import Player
from kalliope.core.Models.Settings import Settings
from kalliope.core.PlayerLauncher import PlayerLauncher


class TestPlayerLauncher(unittest.TestCase):
    """
    Class to test Launchers Classes (PlayerLauncher) and methods
    """

    def setUp(self):
        pass

    ####
    # Player Launcher
    def test_get_player(self):
        """
        Test the PlayerLauncher trying to run the Player
        """
        player1 = Player("Player", {})
        player2 = Player("Player2", {'test': "hitheparamtest"})
        settings = Settings()
        settings.players = [player1, player2]
        with mock.patch("kalliope.core.Utils.get_dynamic_class_instantiation") as mock_get_class_instantiation:
            # Get the player1
            settings.default_player_name = "Player"
            PlayerLauncher.get_player(settings=settings)

            mock_get_class_instantiation.assert_called_once_with(package_name="players",
                                                                 module_name=player1.name,
                                                                 parameters=player1.parameters)
            mock_get_class_instantiation.reset_mock()

            # Get the player 2
            settings.default_player_name = "Player2"
            PlayerLauncher.get_player(settings=settings)

            mock_get_class_instantiation.assert_called_once_with(package_name="players",
                                                                 module_name=player2.name,
                                                                 parameters=player2.parameters)
            mock_get_class_instantiation.reset_mock()


from kalliope.core.Cortex import Cortex
from kalliope.core.Utils import Utils

import logging

logging.basicConfig()
logger = logging.getLogger("kalliope")


class NeuronParameterLoader(object):

    @classmethod
    def get_parameters(cls, synapse_order, user_order):
        """
        Class method to get all params coming from a string order. Returns a dict of key/value.
        """
        params = dict()
        if Utils.is_containing_bracket(synapse_order):
            params = cls._associate_order_params_to_values(user_order, synapse_order)
            logger.debug("[NeuronParameterLoader.get_parameters]Parameters for order: %s" % params)
            # we place the dict of parameters load from order into a cache in Cortex so the user can save it later
            Cortex.add_parameters_from_order(params)
        return params

    @classmethod
    def _associate_order_params_to_values(cls, order, order_to_check):
        """
        Associate the variables from the order to the incoming user order
        :param order_to_check: the order to check incoming from the brain
        :type order_to_check: str
        :param order: the order from user
        :type order: str
        :return: the dict corresponding to the key / value of the params
        """
        logger.debug("[NeuronParameterLoader._associate_order_params_to_values] user order: %s, "
                     "order from synapse: %s" % (order, order_to_check))

        list_word_in_order = Utils.remove_spaces_in_brackets(order_to_check).split()

        # remove sentence before order which are sentences not matching anyway
        truncate_list_word_said = order.split()

        # make dict var:value
        dict_var = dict()
        for idx, ow in enumerate(list_word_in_order):
            if not Utils.is_containing_bracket(ow):
                while truncate_list_word_said and ow.lower() != truncate_list_word_said[0].lower():
                    truncate_list_word_said = truncate_list_word_said[1:]
            else:
                # remove bracket and grab the next value / stop value
                var_name = ow.replace("{{", "").replace("}}", "")
                stop_value = Utils.get_next_value_list(list_word_in_order[idx:])
                if stop_value is None:
                    dict_var[var_name] = " ".join(truncate_list_word_said)
                    break
                for word_said in truncate_list_word_said:
                    if word_said.lower() == stop_value.lower():  # Do not consider the case
                        break
                    if var_name in dict_var:
                        dict_var[var_name] += " " + word_said
                        truncate_list_word_said = truncate_list_word_said[1:]
                    else:
                        dict_var[var_name] = word_said
            truncate_list_word_said = truncate_list_word_said[1:]

        return dict_var



import unittest
import mock

from kalliope.core.NeuronModule import NeuronModule

from kalliope.neurons.systemdate import Systemdate


class TestSystemdate(unittest.TestCase):

    def setUp(self):
        pass

    def test_date_is_returned(self):
        """
        Check that the neuron return consistent values
        :return:
        """

        with mock.patch.object(NeuronModule, 'say', return_value=None) as mock_method:
            systemdate = Systemdate()
            # check returned value
            self.assertTrue(0 <= int(systemdate.message["hours"]) <= 24)
            self.assertTrue(0 <= int(systemdate.message["minutes"]) <= 60)
            self.assertTrue(0 <= int(systemdate.message["weekday"]) <= 6)
            self.assertTrue(1 <= int(systemdate.message["day_month"]) <= 31)
            self.assertTrue(1 <= int(systemdate.message["month"]) <= 12)
            self.assertTrue(2016 <= int(systemdate.message["year"]) <= 3000)


if __name__ == '__main__':
    unittest.main()



import unittest
import ast
import mock

from kalliope.core.Models.Player import Player
from kalliope.core.Models.Signal import Signal
from kalliope.core.Models.RecognitionOptions import RecognitionOptions
from kalliope.core.Models.Tts import Tts

from kalliope.core.Models.Trigger import Trigger

from kalliope.core.Models.Stt import Stt

from kalliope.core.Models.RestAPI import RestAPI

from kalliope.core.Models.Dna import Dna

from kalliope.core import LIFOBuffer
from kalliope.core.Models.Settings import Settings

from kalliope.core.Models import Neuron, Synapse, Brain, Resources, Singleton

from kalliope.core.Models.APIResponse import APIResponse
from kalliope.core.Models.MatchedSynapse import MatchedSynapse


class TestModels(unittest.TestCase):

    def setUp(self):
        # Kill the singleton
        Singleton._instances = dict()

        # Init
        neuron1 = Neuron(name='neurone1', parameters={'var1': 'val1'})
        neuron2 = Neuron(name='neurone2', parameters={'var2': 'val2'})
        neuron3 = Neuron(name='neurone3', parameters={'var3': 'val3'})
        neuron4 = Neuron(name='neurone4', parameters={'var4': 'val4'})

        signal1 = Signal(name="order", parameters="this is the sentence")
        signal2 = Signal(name="order", parameters="this is the second sentence")
        signal3 = Signal(name="order", parameters="that is part of the third sentence")

        self.synapse1 = Synapse(name="Synapse1", neurons=[neuron1, neuron2], signals=[signal1])
        self.synapse2 = Synapse(name="Synapse2", neurons=[neuron3, neuron4], signals=[signal2])
        self.synapse3 = Synapse(name="Synapse3", neurons=[neuron2, neuron4], signals=[signal3])

        self.all_synapse_list1 = [self.synapse1,
                                  self.synapse2,
                                  self.synapse3]

        self.all_synapse_list2 = [self.synapse2,
                                  self.synapse3]

        self.brain_test1 = Brain(synapses=self.all_synapse_list1)
        self.brain_test2 = Brain(synapses=self.all_synapse_list2)
        # this brain is the same as the first one
        self.brain_test3 = Brain(synapses=self.all_synapse_list1)

        self.settings_test = Settings()

        # clean the LiFO
        LIFOBuffer.lifo_list = list()

    def test_APIResponse(self):
        user_order = "user order"
        self.matched_synapse = MatchedSynapse(matched_synapse=self.synapse1, matched_order=user_order)

        api_response = APIResponse()
        api_response.user_order = user_order
        api_response.list_processed_matched_synapse = [self.matched_synapse]

        expected_result_serialize = {
            'status': None,
            'matched_synapses':
                [
                    {
                        'matched_order': 'user order',
                        'neuron_module_list': [],
                        'synapse_name': 'Synapse1'
                    }
                ],
                'user_order': 'user order'
        }

        self.assertDictEqual(expected_result_serialize, api_response.serialize())

    def test_Brain(self):
        # test get synapse by name
        expect_result = self.synapse1
        synapse_name = "Synapse1"
        self.assertEqual(self.brain_test1.get_synapse_by_name(synapse_name), expect_result)

        # test equals
        self.assertTrue(self.brain_test1.__eq__(self.brain_test3))

        # test not equals
        self.assertFalse(self.brain_test1.__eq__(self.brain_test2))

    def test_Dna(self):
        # create DNA object
        dna1 = Dna(name="dna1", module_type="neuron", author="kalliope",
                   kalliope_supported_version="0.4.4", tags="test")

        dna2 = Dna(name="dna2", module_type="neuron", author="community",
                   kalliope_supported_version="0.4.2", tags="other")

        # this dna is exactly the same as the first one
        dna3 = Dna(name="dna1", module_type="neuron", author="kalliope",
                   kalliope_supported_version="0.4.4", tags="test")

        expected_result_serialize = {
            'kalliope_supported_version': '0.4.4',
            'tags': 'test',
            'type': 'neuron',
            'name': 'dna1',
            'author': 'kalliope'
        }

        self.assertDictEqual(expected_result_serialize, dna1.serialize())

        self.assertTrue(dna1.__eq__(dna3))
        self.assertFalse(dna1.__eq__(dna2))

    def test_MatchedSynapse(self):
        user_order = "user order"
        matched_synapse1 = MatchedSynapse(matched_synapse=self.synapse1, matched_order=user_order)
        matched_synapse2 = MatchedSynapse(matched_synapse=self.synapse2, matched_order=user_order)
        matched_synapse3 = MatchedSynapse(matched_synapse=self.synapse1, matched_order=user_order)

        expected_result_serialize = {
            'matched_order': 'user order',
            'neuron_module_list': [],
            'synapse_name': 'Synapse1'
        }

        self.assertDictEqual(expected_result_serialize, matched_synapse1.serialize())

        self.assertTrue(matched_synapse1.__eq__(matched_synapse3))
        self.assertFalse(matched_synapse1.__eq__(matched_synapse2))

        # test neuron parameter loader is called
        with mock.patch("kalliope.core.NeuronParameterLoader.get_parameters") as mock_get_parameters:

            MatchedSynapse(matched_synapse=self.synapse1, matched_order=user_order, user_order=user_order)
            mock_get_parameters.assert_called_once_with(synapse_order=user_order,
                                                        user_order=user_order)
            mock_get_parameters.reset_mock()

    def test_Neuron(self):

        neuron1 = Neuron(name="test", parameters={"key1": "val1", "key2": "val2"})
        neuron2 = Neuron(name="test", parameters={"key3": "val3", "key4": "val4"})
        neuron3 = Neuron(name="test", parameters={"key1": "val1", "key2": "val2"})

        expected_result_serialize = {'name': 'test', 'parameters': {'key2': 'val2', 'key1': 'val1'}}

        self.assertDictEqual(expected_result_serialize, neuron1.serialize())

        self.assertTrue(neuron1.__eq__(neuron3))
        self.assertFalse(neuron1.__eq__(neuron2))

        # test password
        neuron_name = "test"
        neuron_parameters = {
            "password": "my secret",
            "parameter": "test"
        }

        neuron = Neuron()
        neuron.name = neuron_name
        neuron.parameters = neuron_parameters

        expected_result_str = "{'name': 'test', 'parameters': {'password': '*****', 'parameter': 'test'}}"

        self.assertDictEqual(ast.literal_eval(neuron.__str__()), ast.literal_eval(expected_result_str))

        neuron_name = "test"
        neuron_parameters = {
            "password_parameter": "my secret",
            "parameter": "test"
        }

        neuron = Neuron()
        neuron.name = neuron_name
        neuron.parameters = neuron_parameters

        expected_result_str = "{'name': 'test', 'parameters': {'parameter': 'test', 'password_parameter': '*****'}}"

        self.assertDictEqual(ast.literal_eval(neuron.__str__()), ast.literal_eval(expected_result_str))

    def test_Resources(self):
        resource1 = Resources(neuron_folder="/path/neuron", stt_folder="/path/stt",
                              tts_folder="/path/tts", trigger_folder="/path/trigger")

        resource2 = Resources(neuron_folder="/other_path/neuron", stt_folder="/other_path/stt",
                              tts_folder="/other_path/tts", trigger_folder="/other_path/trigger")

        resource3 = Resources(neuron_folder="/path/neuron", stt_folder="/path/stt",
                              tts_folder="/path/tts", trigger_folder="/path/trigger")

        expected_result_serialize = {
            'tts_folder': '/path/tts',
            'neuron_folder': '/path/neuron',
            'stt_folder': '/path/stt',
            'trigger_folder': '/path/trigger',
            'signal_folder': None
        }

        self.assertDictEqual(expected_result_serialize, resource1.serialize())

        self.assertTrue(resource1.__eq__(resource3))
        self.assertFalse(resource1.__eq__(resource2))

    def test_RestAPI(self):

        rest_api1 = RestAPI(password_protected=True, login="admin", password="password", active=True,
                            port=5000, allowed_cors_origin="*")

        rest_api2 = RestAPI(password_protected=False, active=False,
                            port=5000, allowed_cors_origin=None)

        rest_api3 = RestAPI(password_protected=True, login="admin", password="password", active=True,
                            port=5000, allowed_cors_origin="*")

        expected_result_serialize = {
            'password_protected': True,
            'port': 5000,
            'active': True,
            'allowed_cors_origin': '*',
            'password': 'password',
            'login': 'admin'
        }

        self.assertDictEqual(expected_result_serialize, rest_api1.serialize())

        self.assertTrue(rest_api1.__eq__(rest_api3))
        self.assertFalse(rest_api1.__eq__(rest_api2))

    def test_Settings(self):
        with mock.patch('platform.machine', return_value='pumpkins'):
            rest_api1 = RestAPI(password_protected=True,
                                login="admin",
                                password="password",
                                active=True,
                                port=5000, allowed_cors_origin="*")

            recognition_options = RecognitionOptions()

            setting1 = Settings(default_tts_name="pico2wav",
                                default_stt_name="google",
                                default_trigger_name="swoyboy",
                                default_player_name="mplayer",
                                ttss=["ttts"],
                                stts=["stts"],
                                triggers=["snowboy"],
                                players=["mplayer"],
                                rest_api=rest_api1,
                                cache_path="/tmp/kalliope",
                                resources=None,
                                variables={"key1": "val1"},
                                recognition_options=recognition_options,
                                start_options={'muted': False})
            setting1.kalliope_version = "0.4.5"

            setting2 = Settings(default_tts_name="accapela",
                                default_stt_name="bing",
                                default_trigger_name="swoyboy",
                                default_player_name="mplayer",
                                ttss=["ttts"],
                                stts=["stts"],
                                triggers=["snowboy"],
                                rest_api=rest_api1,
                                cache_path="/tmp/kalliope_tmp",
                                resources=None,
                                variables={"key1": "val1"},
                                recognition_options=recognition_options,
                                start_options={'muted': False})
            setting2.kalliope_version = "0.4.5"

            setting3 = Settings(default_tts_name="pico2wav",
                                default_stt_name="google",
                                default_trigger_name="swoyboy",
                                default_player_name="mplayer",
                                ttss=["ttts"],
                                stts=["stts"],
                                triggers=["snowboy"],
                                players=["mplayer"],
                                rest_api=rest_api1,
                                cache_path="/tmp/kalliope",
                                resources=None,
                                variables={"key1": "val1"},
                                recognition_options=recognition_options,
                                start_options={'muted': False})
            setting3.kalliope_version = "0.4.5"

            expected_result_serialize = {
                'default_tts_name': 'pico2wav',
                'hooks': None,
                'rest_api':
                    {
                        'password_protected': True,
                        'port': 5000,
                        'active': True,
                        'allowed_cors_origin': '*',
                        'password': 'password',
                        'login': 'admin'
                    },
                'default_stt_name': 'google',
                'kalliope_version': '0.4.5',
                'default_trigger_name': 'swoyboy',
                'default_player_name': 'mplayer',
                'cache_path': '/tmp/kalliope',
                'stts': ['stts'],
                'machine': 'pumpkins',
                'ttss': ['ttts'],
                'variables': {'key1': 'val1'},
                'resources': None,
                'triggers': ['snowboy'],
                'players': ['mplayer'],
                'recognition_options': {'energy_threshold': 4000, 'adjust_for_ambient_noise_second': 0},
                'start_options': {'muted': False}
            }

            self.maxDiff = None
            self.assertDictEqual(expected_result_serialize, setting1.serialize())

            self.assertTrue(setting1.__eq__(setting3))
            self.assertFalse(setting1.__eq__(setting2))

    def test_Stt(self):
        stt1 = Stt(name="stt1", parameters={"key1": "val1"})
        stt2 = Stt(name="stt2", parameters={"key2": "val2"})
        stt3 = Stt(name="stt1", parameters={"key1": "val1"})

        expected_result_serialize = {'name': 'stt1', 'parameters': {'key1': 'val1'}}

        self.assertDictEqual(expected_result_serialize, stt1.serialize())

        self.assertTrue(stt1.__eq__(stt3))
        self.assertFalse(stt1.__eq__(stt2))

    def test_Synapse(self):
        neuron1 = Neuron(name='neurone1', parameters={'var1': 'val1'})
        neuron2 = Neuron(name='neurone2', parameters={'var2': 'val2'})
        neuron3 = Neuron(name='neurone3', parameters={'var3': 'val3'})
        neuron4 = Neuron(name='neurone4', parameters={'var4': 'val4'})

        signal1 = Signal(name="order", parameters="this is the sentence")
        signal2 = Signal(name="order", parameters="this is the second sentence")

        synapse1 = Synapse(name="Synapse1", neurons=[neuron1, neuron2], signals=[signal1])
        synapse2 = Synapse(name="Synapse2", neurons=[neuron3, neuron4], signals=[signal2])
        synapse3 = Synapse(name="Synapse1", neurons=[neuron1, neuron2], signals=[signal1])

        expected_result_serialize = {
            'signals': [
                {
                    'name': 'order',
                    'parameters': 'this is the sentence'
                }
            ],
            'neurons': [
                {
                    'name': 'neurone1',
                    'parameters': {
                         'var1': 'val1'
                     }
                },
                {
                    'name': 'neurone2',
                    'parameters':
                        {
                            'var2': 'val2'
                        }
                }
            ],
            'name': 'Synapse1'
        }

        self.assertDictEqual(expected_result_serialize, synapse1.serialize())

        self.assertTrue(synapse1.__eq__(synapse3))
        self.assertFalse(synapse1.__eq__(synapse2))

    def test_Trigger(self):
        trigger1 = Trigger(name="trigger1", parameters={"key1": "val1"})
        trigger2 = Trigger(name="trigger2", parameters={"key2": "val2"})
        trigger3 = Trigger(name="trigger1", parameters={"key1": "val1"})

        expected_result_serialize = {'name': 'trigger1', 'parameters': {'key1': 'val1'}}

        self.assertDictEqual(expected_result_serialize, trigger1.serialize())

        self.assertTrue(trigger1.__eq__(trigger3))
        self.assertFalse(trigger1.__eq__(trigger2))

    def test_Player(self):
        player1 = Player(name="player1", parameters={"key1": "val1"})
        player2 = Player(name="player2", parameters={"key2": "val2"})
        player3 = Player(name="player1", parameters={"key1": "val1"})

        expected_result_serialize = {'name': 'player1', 'parameters': {'key1': 'val1'}}

        self.assertDictEqual(expected_result_serialize, player1.serialize())

        self.assertTrue(player1.__eq__(player3))
        self.assertFalse(player1.__eq__(player2))

    def test_Tts(self):
        tts1 = Tts(name="tts1", parameters={"key1": "val1"})
        tts2 = Tts(name="tts2", parameters={"key2": "val2"})
        tts3 = Tts(name="tts1", parameters={"key1": "val1"})

        expected_result_serialize = {'name': 'tts1', 'parameters': {'key1': 'val1'}}

        self.assertDictEqual(expected_result_serialize, tts1.serialize())

        self.assertTrue(tts1.__eq__(tts3))
        self.assertFalse(tts1.__eq__(tts2))


if __name__ == '__main__':
    unittest.main()

    # suite = unittest.TestSuite()
    # suite.addTest(TestLIFOBuffer("test_process_neuron_list"))
    # runner = unittest.TextTestRunner()
    # runner.run(suite)



import logging

from kalliope import SignalLauncher
from kalliope.core.NeuronModule import NeuronModule

logging.basicConfig()
logger = logging.getLogger("kalliope")


class Mute(NeuronModule):

    def __init__(self, **kwargs):
        super(Mute, self).__init__(**kwargs)

        self.status = kwargs.get('status', None)

        # check if parameters have been provided
        if self._is_parameters_ok():
            signal_order = SignalLauncher.get_order_instance()
            if signal_order is not None:
                signal_order.set_mute_status(self.status)

    def _is_parameters_ok(self):
        """
        Check if received parameters are ok to perform operations in the neuron
        :return: true if parameters are ok, raise an exception otherwise

        .. raises:: MissingParameterException
        """
        if self.status is None:
            logger.debug("[Mute] You must specify a status with a boolean")
            return False
        return True



import logging

logging.basicConfig()
logger = logging.getLogger("kalliope")


class Topic(object):
    def __init__(self, name=None, synapses=None, is_json=False):
        self.name = name
        self.synapses = synapses
        self.is_json = is_json

    def serialize(self):
        """
        This method allows to serialize in a proper way this object

        :return: A dict of name and parameters
        :rtype: Dict
        """
        return {
            'name': self.name,
            'is_json': self.is_json,
            'synapses': [e.serialize() for e in self.synapses],
        }

    def __str__(self):
        return str(self.serialize())

    def __eq__(self, other):
        """
        This is used to compare 2 objects
        :param other:
        :return:
        """
        return self.__dict__ == other.__dict__


class Broker(object):
    def __init__(self, broker_ip=None, topics=None, port=1883, client_id="kalliope", keepalive=60,
                 username=None, password=None, protocol="MQTTv311", ca_cert=None, certfile=None, keyfile=None,
                 tls_insecure=False):
        self.broker_ip = broker_ip
        self.topics = topics
        if self.topics is None:
            self.topics = list()

        # optional value
        self.port = port
        self.client_id = client_id
        self.keepalive = keepalive
        self.username = username
        self.password = password
        self.protocol = protocol
        self.ca_cert = ca_cert
        self.certfile = certfile
        self.keyfile = keyfile
        self.tls_insecure = tls_insecure

    def serialize(self):
        """
        This method allows to serialize in a proper way this object

        :return: A dict of name and parameters
        :rtype: Dict
        """
        return {
            'broker_ip': self.broker_ip,
            'port': self.port,
            'client_id': self.client_id,
            'keepalive': self.keepalive,
            'username': self.username,
            'password': self.password,
            'protocol': self.protocol,
            'ca_cert': self.ca_cert,
            'certfile': self.certfile,
            'keyfile': self.keyfile,
            'tls_insecure': self.tls_insecure,
            'topics': [e.serialize() for e in self.topics],
        }

    def __str__(self):
        return str(self.serialize())

    def build_from_signal_dict(self, dict_parameters):
        """
        Build the Broker object from a received dict of parameters
        :param dict_parameters: dict of parameters used to build the Broker object
        """
        logger.debug("[Broker] Build broker object from received parameters: %s" % dict_parameters)

        self.broker_ip = dict_parameters["broker_ip"]

        if "broker_port" in dict_parameters:
            self.port = dict_parameters["broker_port"]
            # keep alive must be an integer
            if not isinstance(self.keepalive, int):
                try:
                    self.port = int(self.port)
                except ValueError:
                    logger.debug("[Broker] Invalid port value, fallback to 1883")
                    self.port = 1883
        else:
            # set default port
            self.port = 1883

        if "client_id" in dict_parameters:
            self.client_id = dict_parameters["client_id"]
        else:
            self.client_id = "kalliope"

        if "username" in dict_parameters:
            self.username = dict_parameters["username"]

        if "password" in dict_parameters:
            self.password = dict_parameters["password"]

        if "keepalive" in dict_parameters:
            self.keepalive = dict_parameters["keepalive"]
            # keep alive must be an integer
            if not isinstance(self.keepalive, int):
                try:
                    self.keepalive = int(self.keepalive)
                except ValueError:
                    logger.debug("[Broker] Invalid keepalive value, fallback to 60")
                    self.keepalive = 60
        else:
            # set default value
            self.keepalive = 60

        if "protocol" in dict_parameters:
            if dict_parameters["protocol"] not in ["MQTTv31", "MQTTv311"]:
                logger.debug("[Broker] Invalid protocol value, fallback to MQTTv311")
                self.protocol = "MQTTv311"
            else:
                self.protocol = dict_parameters["protocol"]
        else:
            self.protocol = "MQTTv311"

        if "ca_cert" in dict_parameters:
            self.ca_cert = dict_parameters["ca_cert"]

        if "certfile" in dict_parameters:
            self.certfile = dict_parameters["certfile"]

        if "keyfile" in dict_parameters:
            self.keyfile = dict_parameters["keyfile"]

        if "tls_insecure" in dict_parameters:
            self.tls_insecure = dict_parameters["tls_insecure"]
            self.tls_insecure = bool(self.tls_insecure)
        else:
            self.tls_insecure = False

    def __eq__(self, other):
        """
        This is used to compare 2 objects
        :param other:
        :return:
        """
        return self.__dict__ == other.__dict__
